"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const FileSystem_1 = require("./FileSystem");
const logger_1 = require("./logger");
class DataImporter {
    constructor(db) {
        this.db = db;
    }
    importData(config) {
        return __awaiter(this, void 0, void 0, function* () {
            const inputDirectory = config.dataPath;
            const existingCollections = yield this.db.getExistingCollectionsArray();
            const collectionsToImport = this.getCollectionsToImport(inputDirectory, existingCollections);
            for (const collection of collectionsToImport) {
                if (collection.shouldCreate) {
                    logger_1.log(`Creating collection '${collection.name}'...`);
                    yield this.db.createCollection(collection.name);
                }
                yield this.importCollection(collection.name, collection.directoryPath, config);
            }
        });
    }
    getCollectionsToImport(inputDirectory, existingCollections = []) {
        const collectionsDirectories = FileSystem_1.fileSystem.listValidDirectories(inputDirectory);
        const collectionsToImport = collectionsDirectories.map(collectionDirectory => {
            return this.getCollectionToImport(collectionDirectory, existingCollections, inputDirectory);
        });
        return collectionsToImport;
    }
    importCollection(collectionName, collectionPath, config) {
        return __awaiter(this, void 0, void 0, function* () {
            const fileNames = FileSystem_1.fileSystem.listFileNames(collectionPath);
            if (fileNames.length === 0) {
                logger_1.log(`Directory '${collectionPath}' is empty. Skipping...`);
                return;
            }
            const documentFileNames = FileSystem_1.fileSystem.getSupportedDocumentFileNames(fileNames, config.supportedExtensions);
            if (documentFileNames.length === 0) {
                logger_1.log(`No documents found for collection '${collectionName}'. Skipping...`);
                return;
            }
            logger_1.log(`Inserting documents into collection '${collectionName}'...`);
            const documentsContentArray = FileSystem_1.fileSystem.getFilesContentArray(collectionPath, documentFileNames);
            const documentsToInsert = config.replaceIdWithUnderscoreId
                ? this.replaceDocumentIdWithUnderscoreId(documentsContentArray)
                : documentsContentArray;
            this.db.insertDocumentsIntoCollection(documentsToInsert, collectionName);
        });
    }
    getCollectionToImport(collectionDirectory, existingCollections, inputDirectory) {
        const collectionName = this.getCollectionName(collectionDirectory);
        return {
            name: collectionName,
            shouldCreate: this.shouldCreateCollection(collectionName, existingCollections),
            directoryName: collectionDirectory,
            directoryPath: `${inputDirectory}/${collectionDirectory}`,
        };
    }
    shouldCreateCollection(collection, existingCollections = []) {
        return existingCollections.indexOf(collection) === -1;
    }
    getCollectionName(directoryName) {
        // Directory name pattern: {import order}-{collection name}
        // TODO: Use Regex and allow more separators
        let collectionName;
        if (directoryName.includes(DataImporter.DIRECTORY_NAME_PATTERN_SEPARATOR)) {
            collectionName = directoryName.split(DataImporter.DIRECTORY_NAME_PATTERN_SEPARATOR)[1];
        }
        else {
            collectionName = directoryName;
        }
        return collectionName;
    }
    replaceDocumentIdWithUnderscoreId(documents) {
        return documents.map(document => {
            if (typeof document.id === 'undefined') {
                return document;
            }
            const documentToInsert = Object.assign({}, document, { _id: document.id });
            delete documentToInsert.id;
            return documentToInsert;
        });
    }
}
DataImporter.DIRECTORY_NAME_PATTERN_SEPARATOR = '-';
exports.DataImporter = DataImporter;
//# sourceMappingURL=DataImporter.js.map